from bzv.yearnormalize import YearNormalize
from urllib import urlencode
from re import compile

urnRe = compile(r'^(?i)urn:(\w+):([\w-]+)$')

def xpath(node, path):
    return node.xpath(path, {'mods':'http://www.loc.gov/mods/v3'})
def xpathString(node, path):
    return ''.join(xpath(node, path)[:1])

def stripUrn(genreString):
    result = genreString.split('/')[-1]
    for allowed in ['article', 'preprint', 'proceeding', 'bookitem', 'book', 'journal', 'conference']:
        if allowed in result:
            return allowed
    return ''

def getURN(identifierString):
    m = urnRe.match(identifierString)
    if not m:
        return '', ''
    type, identifier = m.groups()
    return type.lower(), identifier
    

def add(aDictionary, key, value):
    if value:
        aDictionary[key] = value
    
class OpenURL(object):
    def __init__(self, baseurl):
        self._baseurl = baseurl

    def createOpenURL(self, lxmlNode):
        return '%s?%s' % (self._baseurl, self._create(lxmlNode))
        
    def _create(self, lxmlNode):
        data = {}
        add(data, 'genre', stripUrn(xpathString(lxmlNode, 'mods:genre/text()')))
        add(data, 'aulast', xpathString(lxmlNode, 'mods:name[@type="personal"][mods:role/mods:roleTerm="aut"]/mods:namePart[@type="family"]/text()'))
        add(data, 'aufirst', xpathString(lxmlNode, 'mods:name[@type="personal"][mods:role/mods:roleTerm="aut"]/mods:namePart[@type="given"]/text()').rstrip('.'))
        add(data, 'atitle', xpathString(lxmlNode, 'mods:titleInfo/mods:title/text()'))

        identifier = xpathString(lxmlNode, 'mods:relatedItem[@type="host"]/mods:identifier/text()')
        typeIdentifier, identifier = getURN(identifier)
        if typeIdentifier in ['issn', 'isbn']:
            add(data, typeIdentifier, identifier.replace('-',''))
        add(data, 'title', xpathString(lxmlNode, 'mods:relatedItem[@type="host"]/mods:titleInfo/mods:title/text()'))
        volume = xpathString(lxmlNode, 'mods:relatedItem[@type="host"]/mods:part/mods:detail[@type="volume"]/mods:number/text()')
	add(data, 'volume', volume.split('-')[0])
        issue = xpathString(lxmlNode, 'mods:relatedItem[@type="host"]/mods:part/mods:detail[@type="issue"]/mods:number/text()')
	add(data, 'issue', issue.split('-')[0])
        spage = xpathString(lxmlNode, 'mods:relatedItem[@type="host"]/mods:part/mods:extent[@unit="page"]/mods:start/text()')
        epage = xpathString(lxmlNode, 'mods:relatedItem[@type="host"]/mods:part/mods:extent[@unit="page"]/mods:end/text()')
        add(data, 'spage', spage)
        add(data, 'epage', epage)
        if spage and epage:
            add(data, 'pages', '%s-%s' % (spage, epage))
        add(data, 'date',  YearNormalize(yearRange=(1000,2100)).normalize(xpathString(lxmlNode, 'mods:originInfo/mods:dateIssued/text()')))
#        add(data, 'location', xpathString(lxmlNode, 'mods:location/mods:holdingSimple/mods:copyInformation/mods:shelfLocator/text()'))

        #'auinit'
        #'auinit1'
        #'auinitm'
        #'eissn'
        #'coden'
        #'sici'
        #'bici'
        #'part'
        
        #'artnum'
        #'date'
        #'ssn'
        #'quarter'

        return urlencode(data)
